class OsmSign < ActiveRecord::Base 
  belongs_to :osm_trail
  belongs_to :osm_guidepost

  delegate :kctref, :exists_by_any_ref?, :guidepost, :osmid, to: :osm_guidepost
  delegate :trail, to: :osm_trail

  def km;          super || km_on_trail end
  def km_on_trail; km_on(sorted_osm_trail&.gpx) end
  def km_on(gpx);  gpx&.kilometraz_of_nearest_point_to(self.osm_guidepost.coordinates) end

  def sorted_osm_trail; osm_trail&.sorted end
  def exists?; !sign.nil? end

  def sign
    @sign ||= unless guidepost.nil?
      Sign.where(guidepost: guidepost, trail: trail).first
    else
      nil
    end
  end
  
  def kct_sign
    unless sign.nil? or sign.kct_guidepost.nil?
      sign.kct_sign
    else # guess by refs
      KctKilometraz.where(
        :kct_trail_id => KctTrail.find_by_kctref(osm_trail.kctref), 
        :kct_guidepost_id => KctGuidepost.find_by_kctref(kctref) 
      ).first
    end
  end
  
  # estimates my position on trail in kms by averaging distance from other signs with known kct km values
  def estimated_km_from_kct_kms
    osm_master_signs = self.osm_trail.osm_signs.select {|os| os.exists?}  # these are the other signs on trail which i will be using to guess the position
    estimated_kct_kms = osm_master_signs.collect do |osm_master_sign|
      if kct_master_sign = KctKilometraz.where(
                             :kct_trail_id => self.osm_trail.kct_trail.id,
                             :kct_guidepost => osm_master_sign.guidepost.kct_guidepost 
                           ).first
        kct_master_sign.km + self.osm_trail.trail_relative_distance(osm_master_sign,self)
      else
        nil
      end
    end
    estimated_kct_kms.compact!
    return estimated_kct_kms.inject(0.0) { |sum, el| sum + el } / estimated_kct_kms.size
  end
  
end
